#!/bin/bash
# Note to Developers:
# search for the CUSTOMIZATION SECTION to make changes to what gets installed


STATUS_WARN=0
STATUS_ERR=0

MODE="install"

Log() { echo $* ; }

LogError()
{
    Log ERROR: $*
    STATUS_ERR=1
}

LogWarning()
{
    Log WARNING: $*
    STATUS_WARN=1
}

# ProvG Helper Functions
KillDaemon()
{
  if ps -e | grep issDaemon >/dev/null 2>&1; then
    Log "Stopping ISS Daemon."
    /etc/init.d/issDaemon stop >/dev/null 2>&1
  fi
}

Install()
{

    KillDaemon

    INSTALL_LIST=
    for i in $@ ; do
        PKGVER=`rpm -qp $i`
        BASE=`basename $i -new.rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTVER=`rpm -q $BASE`
            if iss-rpm-compare $i ; then
                Log Upgrading $INSTVER to $PKGVER
                if [ -f $BASE-new.sigtgz ] ; then 
                    tar -C / -xzf $BASE-new.sigtgz || LogError Failed to extract signatures for $BASE
                fi
                INSTALL_LIST="$i $INSTALL_LIST"
            else
                LogWarning Skipping $PKGVER, $INSTVER already installed
            fi
        else
            Log Installing $PKGVER
            if [ -f $BASE-new.sigtgz ] ; then
                tar -C / -xzf $BASE-new.sigtgz || LogError Failed to extract signatures for $BASE
            fi
            INSTALL_LIST="$i $INSTALL_LIST"
        fi
    done
    if [ ! -z "$INSTALL_LIST" ] ; then
        rpm $RPM_INSTALL_OPTS $INSTALL_LIST && return 0 || LogError Failed while installing packages $INSTALL_LIST
    fi
    return 1
}

Uninstall()
{

    KillDaemon

    INSTALL_LIST=
    for i in $@ ; do
        PKGVER=`rpm -qp $i`
        BASE=`basename $i -original.rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTVER=`rpm -q $BASE`
            #if iss-rpm-compare $i ; then
                Log Downgrading $INSTVER to $PKGVER
                if [ -f $BASE-original.sigtgz ] ; then
                    tar -C / -xzf $BASE-original.sigtgz || LogError Failed to extract signatures for $BASE
                fi
                INSTALL_LIST="$i $INSTALL_LIST"
            #else
                #LogWarning Skipping $PKGVER, $INSTVER already installed
            #fi
        else
            Log Installing $PKGVER
            if [ -f $BASE-original.sigtgz ] ; then
                tar -C / -xzf $BASE-original.sigtgz || LogError Failed to extract signatures for $BASE
            fi
            INSTALL_LIST="$i $INSTALL_LIST"
        fi
    done
    if [ ! -z "$INSTALL_LIST" ] ; then
        rpm $RPM_UNINSTALL_OPTS $INSTALL_LIST && return 0 || LogError Failed while installing packages $INSTALL_LIST
    fi
    return 1
}

CondInstall()
{
    INSTALL_LIST=
    for i in $@ ; do
        BASE=`basename $i .rpm`
        if rpm -q $BASE >/dev/null 2>&1 ; then
            INSTALL_LIST="$i $INSTALL_LIST"
        else
            Log Skipping $i, conditional package was not installed
        fi
    done
    Install $INSTALL_LIST
    return $?
}

RunScript()
{
    if [ -f "$1" ]; then
        chmod +x $1
        ./${1} || LogError script $1 failed
        Log Script $1 completed
    else
        LogError Missing script $1
        return $?
    fi
}

Log Hotfix Installation Started
echo ""

RPM_INSTALL_OPTS="-Uvh"

RPM_UNINSTALL_OPTS="--force -Uvh"

# BEGIN CUSTOMIZATION SECTION

TARGET_XML=/etc/crm/rsPostLocalProperties.xml
TEMP_XML=/tmp/temprsPostLocalProperties.xml
XSLT_UPDATER=rsPostLocalProperties_update.xslt
XSLT_REVERTER=rsPostLocalProperties_revert.xslt
XSLT_VALIDATOR=rsPostLocalProperties_test.xslt
NEWTIMEOUT=300000
NEWNOCACHE=false
OLDTIMEOUT=0
OLDNOCACHE=true

UpdatePostLocalConfig ()
{
        KillDaemon
        #backup target
        cp -f $TARGET_XML $TARGET_XML.pre.bak
        #do transform
        Xalan -o $TEMP_XML $TARGET_XML $XSLT_UPDATER
        #evaluate return code
        if [ $? -eq 0 ]; then
                #test output
                eval $(echo `Xalan "$TEMP_XML" "$XSLT_VALIDATOR" | awk '/The transferTimeout setting/{timeout=$5} /The eventQueueWriteThrough/{nocache=$5} END{print "export timeout="timeout" nocache="nocache
}'`)
                if [ "$timeout" == "$NEWTIMEOUT" ] && [ "$nocache" == "$NEWNOCACHE" ]; then
                        cp -f $TEMP_XML $TARGET_XML
                else
                        LogError Failed to read correct values from translated xml.
                        cp -f $TARGET_XML.pre.bak $TARGET_XML
                fi
        else
                LogError Failed to translate xml.
                cp -f $TARGET_XML.pre.bak $TARGET_XML
        fi
}

RevertPostLocalConfig ()
{
        KillDaemon
        #backup target
        cp -f $TARGET_XML $TARGET_XML.pre.bak
        #do transform
        Xalan -o $TEMP_XML $TARGET_XML $XSLT_REVERTER
        #evaluate return code
        if [ $? -eq 0 ]; then
                #test output
                eval $(echo `Xalan "$TEMP_XML" "$XSLT_VALIDATOR" | awk '/The transferTimeout setting/{timeout=$5} /The eventQueueWriteThrough/{nocache=$5} END{print "export timeout="timeout" nocache="nocache
}'`)
                if [ "$timeout" == "$OLDTIMEOUT" ] && [ "$nocache" == "$OLDNOCACHE" ]; then
                        cp -f $TEMP_XML $TARGET_XML
                else
                        LogError Failed to read correct values from translated xml.
                        cp -f $TARGET_XML.pre.bak $TARGET_XML
                fi
        else
                LogError Failed to translate xml.
                cp -f $TARGET_XML.pre.bak $TARGET_XML
        fi
}

ShowUsage ()
{
	Log "install.sh [OPTIONS]"
	Log "	-r Rollback to previous settings."
	Log "	-h Show this message."
}

while getopts "rh" cmdopt $*
do
	case "$cmdopt" in
		"h")
		    MODE="usage"
		    break
		    ;;
		"r")
		    MODE="rollback"
		    break
		    ;;
		"*")
		    MODE="badargs"
		    break
		    ;;
	esac
done

if [ $MODE == "usage" ]; then
	ShowUsage
else
	fwversion=`issasApache -a si | grep "Last Firmware Update" | awk '{print $7}'`

	#eval $(echo `issasApache -a si | awk ' /Last Firmware Update/{firm=$7} /Model Number/{model=substr($2,10)} END{print "export fwversion="firm" model="model} '`)

	#echo $fwversion $model
	echo "Checking Firmware Version..."
	if [ -n "$fwversion" ]; then
		if [ $fwversion == 3.2 ] || [ $fwversion == 3.3 ]; then
			if [ $MODE == "install" ]; then
				echo "256 256 1 32" > /proc/sys/vm/lowmem_reserve_ratio
                        	grep "vm.lowmem_reserve_ratio" /etc/sysctl.conf >/dev/null
                        	if [ $? == "1" ]; then
                                	echo "vm.lowmem_reserve_ratio = 256 256 1 32" >> /etc/sysctl.conf
                        	else
                                	LogWarning Previous vm.lowmem_reserve_ratio setting present in sysctl.conf
                                	LogWarning Please manually verify that lowmem_reserve_ratio is properly set
                        	fi
                                UpdatePostLocalConfig
                        elif [ $MODE == "rollback" ]; then
                                RevertPostLocalConfig
				LogWarning "Memory configuration settings (vm.lowmem_reserve_ratio) will NOT be reverted."
                        fi
                        sync
			service issDaemon start
		else
			echo "Incorrect firmware version for this patch. Firmware 3.2 or 3.3 is required. Your firmware version is $fwversion."
			exit 1
		fi
	else
			echo "Please ensure the issDaemon is running prior to installing this patch."
                	exit 1
	fi
fi

#END CUSTOMIZATION SECTION
echo ""
if [ $STATUS_ERR -ne 0 ] ; then
    Log "result_code = failed"
    Log "Hotfix Installation Failed"
    if [ $MODE == "install" ]; then
	Log `date` ": Combined MSL Config and Memory Settings patch failed to install." >> /cache/iss/patchhistory.log
    elif [ $MODE == "rollback" ]; then
	Log `date` ": Combined MSL Config and Memory Settings patch failed to uninstall." >> /cache/iss/patchhistory.log
    fi
elif [ $STATUS_WARN -ne 0 ] ; then
    Log "result_code = warning"
    Log "Hotfix installation completed with warnings"
    if [ $MODE == "install" ]; then
	Log `date` ": Combined MSL Config and Memory Settings patch installed with warnings." >> /cache/iss/patchhistory.log
    elif [ $MODE == "rollback" ]; then
	Log `date` ": Combined MSL Config and Memory Settings patch uninstalled with warnings." >> /cache/iss/patchhistory.log
    fi
else
    Log "result_code = success"
    Log "Hotfix Installation Completed Successfully"
    if [ $MODE == "install" ]; then
	Log `date` ": Combined MSL Config and Memory Settings patch successfully installed." >> /cache/iss/patchhistory.log
    elif [ $MODE == "rollback" ]; then
	Log `date` ": Combined MSL Config and Memory Settings patch successfully uninstalled." >> /cache/iss/patchhistory.log
    fi
fi
echo""
echo ""
